<?php


/*
 * ***********************************************************************************************
 * Filename:     album.class.php
 * Module:       Album Class
 * Subcategory:  Main Album Management
 * Description:  central class of album management and hitlog
 * ***********************************************************************************************
 *
 *
 * Project:     yappa-ng : yet another php photo album - next generation
 * Author:      Fritz Berger <wizard@zirkon.at>
 * Copyright:   2003 Fritz Berger
 * $Header: /cvsroot/yappa-ng/yappa-ng/album.class.php,v 1.28 2003/08/14 23:41:39 adoll Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You may contact the author of yappa-ng by e-mail at:
 *    wizard@zirkon.at
 *
 * The latest version of yappa-ng can be obtained from:
 *    http://www.zirkon.at/zirkon/scripts/yappa-ng/yappa-ng_main_ger.html (german)
 *    http://www.zirkon.at/zirkon/scripts/yappa-ng/yappa-ng_main_eng.html (english)
 *
 * ***********************************************************************************************
 * ***********************************************************************************************
 * ***                                                                                         ***
 * ***   yappa-ng is based on "YAPPA v1.7 devel - 22.April 2002"                               ***
 * ***                         released on http://sourceforge.net/projects/yappa/              ***
 * ***   YAPPA is Copyright Federico 'pix' Feroldi (pix@pixzone.com)                           ***
 * ***   YAPPA is released under the GNU GPL                                                   ***
 * ***                                                                                         ***
 * ***********************************************************************************************
 * ***********************************************************************************************
 */


// include_once("defines.inc.php");
include_once("common.php");
include_once("image.class.php");
include_once(singleslash($config['path_lang_include'] . "language_selection.inc.php"));

/*
 * class AlbumGroup
 * This class manages an album collection (album group), that is the
 * albums list on the homepage.
 *
 */
 
class AlbumGroup 
{
	var $_group_root, $_albums;

	function AlbumGroup($group_root)
	{
		$this->_group_root = $group_root;
	}

	function exists($album_name)
	{
		return (!(preg_match('/\.\./', $album_name)) && @is_dir($this->_group_root . '/' . $album_name));
	}

	function get_album($album_name)
	{
		return new Album($this->_group_root, $album_name);
	}
}

/*
 * class Album
 * manages an image collection (album)
 *
 */
 
class Album
{
	var $_album_root, $_album_name, $_album_title, $_album_comment;
	var $_mtime;
	var $_preview;

	// Private: Sub-albums of the album.
	// Loaded on demand, NULL indicates the subalbums have not
	// been loaded yet, if there are no subalbums then the value
	// will be an array of count 0.
	// See: get_subalbums()
	var $_subalbums = NULL;

	// Private: Images in the album.
	// Loaded on demand, NULL indicates the images have not
	// been loaded yet, if there are no images then the value
	// will be an array of count 0.
	// See: get_images()
	var $_images = NULL;

	// Private: Captions for the images in the album.
	// Loaded on demand, NULL indicates the captions have not
	// been loaded yet, if there are no captions then the value
	// will be an array of count 0.
	// The key is the image name, the value is the caption.
	// See: get_captions(), get_caption()
	var $_caption = NULL;

	function Album($group_root, $album_name)
	{
		$this->_album_root = $group_root . '/' . $album_name;
		$this->_album_name = $album_name;
	}

	/* reads the album directory and information files */
	function read_dir($depth= 1)
	{
		global $config, $thumbnail_preview;
        
		$this->_images = array();
        
		/* read images */
		$d = dir($this->_album_root);
		while($entry = $d->read()) {
			if(($entry != "..") && ($entry != ".")) {
				$entry_path = $this->_album_root . '/' . $entry;
				if(is_file($entry_path) && get_image_mime($entry_path)) {
					$this->_images[] = $entry;
					$this->_mtime[$entry] = filemtime($this->_album_root . '/' . $entry);
				} elseif(is_dir($entry_path) && (substr($entry, 0, 1) != '.') && (substr($entry, 0, 1) != '_')) {
					if(file_exists($entry_path . "/config_album.inc.php")) {
						$subalbum = new Album($this->_album_root, $entry);
						//if ($depth) {
							$subalbum->read_dir(0);
							$subalbum->read_album_info();
							$this->_subalbums[$entry] = array(
								'name' => $entry,
								'path' => $this->_album_name . '/' . $entry,
								'images' => $subalbum->count_images(),
								'subalbums' => $subalbum->count_subalbums(),
								'title' => $subalbum->_album_title,
								);
							// BUGFIX sent by 'P6A'
							// on some systems the rand(0, 0) gave an error (=only one image in the album)
							if($subalbum->count_images() > 0) {
								if($subalbum->count_images() == 1) {
									$this->_subalbums[$entry]["preview"] = $subalbum->get_imagename(0);
									$this->_preview = $subalbum->get_imagename(0);
								} else {
									$this->_subalbums[$entry]["preview"] = $subalbum->get_imagename(rand(0, ($subalbum->count_images()) - 1));
									$this->_preview = $subalbum->get_imagename(rand(0, ($subalbum->count_images()) - 1));
								}
							}
						//}
					}
				}
			}
		}

		if($config["album_sort_by"] == "name") {
			if($config["album_sort_order"] == "a") {
				@sort($this->_subalbums);
			} else {
				@rsort($this->_subalbums);
			}
		} elseif($config["album_sort_by"] == "mtime") {
			if($config["album_sort_order"] == "a") {
				@asort($this->_subalbums);
			} else {
				@arsort($this->_subalbums);
			}
		}

		return count($this->_images);
	}

	/* sort the images in the selected sortorder
	 * First the 'sort_by', than the 'sort_order'.
	 * The complete image-list (created from the directory-filelisting, meaning it is complete)
	 * is stored in the $this->_images - array.
	 * This is the source array for all other image-releted functions (as sorted here) */
	function sort_dir()
	{
		global $config_album, $config;
		
		/* sorting needed only if there are more than 1 image ! */
		if(@count($this->_images) > 1) {

			/* sorts images based on the filename */
			if($config_album['sort_by'] == "filename") {
				if($config_album['sort_order'] == "a") {
					sort($this->_images);
				} else {
					rsort($this->_images);
				}
			/* sort images based on the 'mtime' */
			} elseif($config_album['sort_by'] == "mtime") {
				if($config_album['sort_order'] == "a") {
					asort($this->_mtime);
				} else {
					arsort($this->_mtime);
				}
				$this->_images = array_keys($this->_mtime);
			/* sort images based on the 'image-hits'.
			* The image-hits are stored in the hits.log in the album-directory.
			* Image-hits are created only if there is a hit - so there could be
			* images which are in the directory, but never hit: they have no entry.
			* On the other hand if you delete an image without the admin-deleteimage-module
			* (from the shell) you may have an image in the image-hitlist which does
			* not exist anymore. */
			} elseif($config_album['sort_by'] == "hits") {
				/* remember ALL images in this album */
				$l_storedirimagelist = $this->_images;
				/* now sort based on the hits.log */
				$l_logfilename = $this->_album_root . "/" . $config['log_filename'];
				$l_hitcounter = new HitCounter($l_logfilename);
				$l_sortedlist = $l_hitcounter->sort_hits();
			/* sort images on the 'image-caption-texts'.
			* The image-captions are stored in the captions.txt in the album-directory.
			* image-captions have to be created - so if you upload an image and dont give
			* it an caption the image is not in this file - you would not see this images!
			* On the other hand if you delete an image without the admin-imagedelete-module
			* (from the shell) you may have an image in the captions.txt which does
			* not exist anymore. */
			} elseif($config_album['sort_by'] == "captions") {
				/* remember ALL images in this album */
				$l_storedirimagelist = $this->_images;
				/* read the image-captions */
				$this->read_images_info();
				$l_sortarray = $this->_caption;
				/* check if there are ANY captions!
				* if not - don't sort! :-) */
				if (count($l_sortarray) > 0 ) {
					/* to sort effectively convert ALL captions to upper case */
					$l_keys = array_keys($l_sortarray);
					foreach($l_keys as $l_up) {
						$l_sortarray[$l_up] = strtoupper($l_sortarray[$l_up]);
					}
					/* now sort based on the captions.txt */
					if (count($l_sortarray) > 1) {
						asort($l_sortarray);
					}
					/* make the list one-dimensional - we need only the filenames - NOT the captions */
					$l_sortedlist = array_keys($l_sortarray);
				}
			}
			if (($config_album['sort_by'] == "hits") || ($config_album['sort_by'] == "captions")) {
				/* First make sure the $l_sortedlist is not zero */
				if (count($l_sortedlist) != 0 ) {
					/* For these cases we now have the '$l_sortedlist' which is sorted on what we have.
					* This list may now have:
					* a) too many images (these entries would create errors and broken links!)
					* b) not all images (the missing images would NOT be shown!)
					* So make sure you have all existing - but only the existing - images!
					* Take the sorted list (criteria) and check every image
					* if it does exist physically and put it into a helparray */
					foreach($l_sortedlist as $l_insorted) {
						foreach($l_storedirimagelist as $l_inlist) {
							if ($l_insorted == $l_inlist) {
								/* in this helparray you have the right sortorder and only existing images */
								$l_helparray[] = $l_insorted;
							}
						}
					}
					$this->_images = $l_helparray;
					/* you now have an helparray where only 'real' images are stored and are
					* in the right sortorder.
					* Now fill in all missing images, which are on the server but not in the list, to the end of the list */
					foreach($l_storedirimagelist as $l_inlist) {
						$l_flag = 0;
						foreach($l_helparray as $l_help) {
							if ($l_inlist == $l_help) {
								$l_flag = 1;
							}
						}
						if ($l_flag != 1) {
							$this->_images[] = $l_inlist;
						}
					}
					/* and now we have the '$this->_images array' complete
					* only thing left to do: check if 'ascending' or 'descending' sortorder is selected 
					* but only if the imagecount > 0 */
					if($config_album['sort_order'] == "a") {
						array_reverse($this->_images);
					}
				}
			}
		}
	}


	function read_album_info()
	{
		global $config;
		global $lang_akt;

		/* check for info file */
		$this->_album_comment = Null;
		$info_filename = $this->_album_root . '/' . $lang_akt["info"];
		if(file_exists($info_filename) && is_file($info_filename) && is_readable($info_filename)) {
			if($fd = fopen($info_filename, "rt")) {
				$this->_album_title = fgets($fd, 1024);
				while(!feof($fd)) {
					$this->_album_comment .= fgets($fd, 1024);
				}
				fclose($fd);
			}
		} elseif ($config['lang_intuitive'] != "no") {
			$info_filename = $this->_album_root . '/' . $lang_akt["info_def"];
			if(file_exists($info_filename) && is_file($info_filename) && is_readable($info_filename)) {
				if($fd = fopen($info_filename, "rt")) {
					$this->_album_title = fgets($fd, 1024);
					while(!feof($fd)) {
						$this->_album_comment .= fgets($fd, 1024);
					}
					fclose($fd);
				}
			}
		}

	}
	/* needed for reread of album comment in admin_module_edit.inc.php !! */
	function clear_album_comment()
	{
		$this->_album_comment = "";
		$this->_album_title = "";
	}

	function read_images_info()
	{
		global $config;
		global $lang_akt;

		/* check for captions file */
		$captions_filename = $this->_album_root . '/' . $lang_akt["captions"];
		if(file_exists($captions_filename) && is_file($captions_filename) && is_readable($captions_filename)) {
			if($fd = fopen($captions_filename, "rt")) {
				while(!feof($fd)) {
					$data = fgets($fd, 1024);
					if(preg_match("/([^\|]+)\|(.+)/", $data, $matches)) {
						$this->_caption[$matches[1]] = $matches[2];
					}
				}
				fclose($fd);
			}
		} elseif ($config['lang_intuitive'] != "no") {
			if (isset($lang_akt["captions_def"])) {
				$l_captions_def = $lang_akt["captions_def"];
			} else {
				$l_captions_def = "";
			}
			$captions_filename = $this->_album_root . '/' . $l_captions_def;
			if(file_exists($captions_filename) && is_file($captions_filename) && is_readable($captions_filename)) {
				if($fd = fopen($captions_filename, "rt")) {
					while(!feof($fd)) {
						$data = fgets($fd, 1024);
						if(preg_match("/([^\|]+)\|(.+)/", $data, $matches)) {
							$this->_caption[$matches[1]] = $matches[2];
						}
					}
					fclose($fd);
				}
			}
		}
	}

	/* needed for reread of image caaptions in admin_module_captions.inc.php !! */
	function clear_images_info()
	{
		$l_arrImages = array();
		$l_arrImages = $this->_images;
		foreach ($l_arrImages as $imagename) {
			$this->_caption[$imagename] = "";
		}
	}


	function read_passwd_info()
	{
		global $passwd_needed;
		global $passwd_txt;

		/* check for password file */
		$passwd_filename = $this->_album_root . "/passwd_txt.inc.php";
		if(file_exists($passwd_filename) && is_file($passwd_filename) && is_readable($passwd_filename)) {
			include($passwd_filename);
			$passwd_needed = 1;
            	} else {
			$passwd_needed = 0;
		}
	}

	function passwd_check($subalbum)
	{
		/* check for password file */
		$passwd_filename = singleslash($subalbum . "/passwd_txt.inc.php");
		if(file_exists($passwd_filename) && is_file($passwd_filename) && is_readable($passwd_filename)) {
			$passwd_check = 1;
            	} else {
			$passwd_check = 0;
		}
		return $passwd_check;
	}

	function read_report_info()
	{
		global $report_enable;
		global $report;
		global $lang_akt;

		/* check forreport file */
		$report_filename = $this->_album_root . "/" . $lang_akt["report"];
		if(file_exists($report_filename) && is_file($report_filename) && is_readable($report_filename)) {
			include($report_filename);
			$report_enable = 1;
            	} else {
			$report_enable = 0;
		}
	}


	/* returns an array with all the images */
	function get_image_array() 
	{
		return $this->_images;
	}
	
	/* returns an array with some informations about this album */
	function get_info() {
 		return array(
			"mtime" => filemtime($this->_album_root),
			"images_count" => count($this->_images),
			"title" => $this->_album_title,
			"comment" => $this->_album_comment,
		);
	}

	/* returns thumbnail information for an image */
	function get_thumbnail($image_name)
	{
		global $config;

		$l_thumb_path = "_" . $config['thumb_width'] . "x" . $config['thumb_height'];
		//$thumbnail_path = singleslash($this->_album_root . '/' . $define["thumbnails_dirname"] . '/' . $image_name);
		$thumbnail_path = singleslash($config['cache_root'] . "/" . $this->_album_name . '/' . $l_thumb_path . '/' . $image_name);

		if(file_exists($thumbnail_path)) {
			$image_size = getimagesize($thumbnail_path);
			return array(
				"width" => $image_size[0],
				"height" => $image_size[1],
				"type" => $image_size[2],
				"html" => $image_size[3],
				"mtime" => filemtime($this->_album_root . '/' . $image_name),
				"caption" => $this->_caption[$image_name],
			);
		} else {
			return array(
				"width" => "0",
				"height" => "0",
				"type" => "",
				"html" => "",
				"mtime" => filemtime($this->_album_root . '/' . $image_name),
				"caption" => $this->_caption[$image_name],
			);
		}
	}

	/* returns information for an image */
	function get_image($image_name)
	{
		$image_path = singleslash($this->_album_root . '/' . $image_name);
		$image_size = getimagesize($image_path);
		$mime = get_image_mime($image_path);
		return array(
			"width" => $image_size[0],
			"height" => $image_size[1],
			"type" => $image_size[2],
			"html" => $image_size[3],
			"mtime" => filemtime($image_path),
			"path" => $image_path,
			"caption" => $this->_caption[$image_name],
			"exif" => (($mime == "jpeg") && function_exists("read_exif_data")) ? read_exif_data($image_path) : '',
			"mime" => $mime,
		);
	}
	
	/* returns previous and next images from an image */
	function get_prevnext($image_name) 
	{
		$prev = "";
		reset($this->_images);
 		while (list ($key, $curr) = each ($this->_images)) {
			if($curr == $image_name) {
				break;
			}
			$prev = $curr;
		}
		return array(
			$prev,		
			current($this->_images)
		);
	}
    
	/* returns the index for an image */
	function get_imageindex($image_name)
	{
		global $count;

		for($count = 0; $count < count($this->_images); $count++) {
			if($this->_images[$count] == $image_name) {
				return $count;
			}
	        }
		/* return -1; */
	}


	function get_imagename($index)
	{
		return $this->_images[$index];
	}

	function image_exists($image_name)
	{
		return in_array($image_name, $this->_images);
	}

	function count_images()
	{
		return count($this->_images);
	}

	function count_subalbums()
	{
		return count($this->_subalbums);
	}

	// Return the images in the album.
	// No images are returned as an array of count 0.
	function get_images()
	  {
	    if ($this->_images == NULL) {
	      $this->read_dir();
	      $this->sort_dir();
	    }
	    return $this->_images;
	  }

	// Return the subalbums of the album.
	// No sub-albums are returned as an array of count 0.
	function get_subalbums()
	  {
	    if ($this->_subalbums == NULL) {
	      $this->read_dir();
	      $this->sort_dir();
	    }
	    return $this->_subalbums;
	  }

	// Return the captions in the album.
	// No images are returned as an array of count 0.
	function get_captions()
	  {
	    if ($this->_caption == NULL) {
	      $this->read_images_info();
	    }
	    return $this->_caption;
	  }

	// Return the caption for an image.
	function get_caption($image_name)
	  {
	    $captions = $this->get_captions();
	    $result = $captions[$image_name];
	    return $result;
	  }

}

/*
 * HitCounter class, store a hits log, an entry is automatically created when
 * passed to add_hit()
 *
 */
class HitCounter
{
	var $_log_filename, $_log_array = array(), $_top10_array = array();
		var $i, $key, $_sortarry = array();

	function HitCounter($log_filename)
	{
		$this->_log_filename = $log_filename;
		if(!file_exists($this->_log_filename)) {
			$this->_write_log();
		}
	}

	function _read_log() 
	{
		if(!file_exists($this->_log_filename) || !is_readable($this->_log_filename) || !$fd = fopen($this->_log_filename, "r")) {
			return;
		}
		flock($fd, 1); // get a shared lock
		$data = fread($fd, filesize($this->_log_filename));
		flock($fd, 3); // release the lock
		fclose($fd);
		$this->_log_array = unserialize( stripslashes($data)); // stripslashes to correct possible 'magic quotes' problem on servers
	}
	
	function _write_log() 
	{
		if(!file_exists($this->_log_filename) || !is_writeable($this->_log_filename) || !$fd = fopen($this->_log_filename, "w")) {
 			return;
		}
		flock($fd, 2); // get an exclusive lock
		fwrite($fd, serialize($this->_log_array));
		flock($fd, 3); // release the lock
		fclose($fd);
	}
	
	function add_hit($keyword)
	{
		$this->_read_log();
		$this->_log_array[$keyword] = floor($this->_log_array[$keyword] + 1);
		$count = $this->_log_array[$keyword];
		$this->_write_log();
		return $count;
	}

	function del_hit($keyword)
	{
		$this->_read_log();
		unset ($this->_log_array[$keyword]);
		$this->_write_log();
	}
	
	function read_hit($keyword)
	{
		$this->_read_log();
		if (isset($this->_log_array[$keyword])) {
			return $this->_log_array[$keyword];
		} else {
			return;
		}
	}

	/* clean the file 'hits.log'.
	 * The image-hits are stored in the hits.log in the album-directory.
	 * Image-hits are created only if there is a hit - so there could be
	 * images which are in the directory, but never hit: they have no entry.
	 * On the other hand if you delete an image without the admin-deleteimage-module
	 * (from the shell or ftp) you may have an image in the image-hitlist which does
	 * not exist anymore. 
	 * These nonexisting images are eliminated here -> thus the name clean_hitslog
	 */
	function clean_hitslog($images_array)
	{
		global $hitlog_readable;
		if ($hitlog_readable) {
			/* initialize needed arrays */
			$l_hit_array_keys = array();
			$l_hit_array_full = array();
			$l_new_array = array();
			/* read the hits.log */
			$this->_read_log();
			/* save the log_array, but only the keys */
			$l_hit_array_full = $this->_log_array;
			$l_hit_array_keys = array_keys($this->_log_array);
			//$l_hit_array_keys = array_keys($l_hit_array_full);
			/* now check all hits if the corresponding images do exist */
			foreach($l_hit_array_keys as $known_hit) {
				foreach($images_array as $real_image) {
					if($known_hit == $real_image) {
						/* image exists - we keep this hitcounter */
						$l_new_array[$known_hit] = $l_hit_array_full[$known_hit];
					}
				}
			}
			/* now restore the cleaned array to the log_array and write the new hits.log */
			$this->_log_array = $l_new_array;
			$this->_write_log();
		}
	}

	function top10()
	{
		global $config_album;
		global $hitlog_readable;

		if ($hitlog_readable) {
		/* abfrage, ob top10 definiert und belegt ist - ansonst Wert 10 */
		$this->_read_log();
		$_sortarray= array();
		$_sortarray = $this->_log_array;
		if (count($_sortarray) > 1) {
			arsort($_sortarray);
		}
		if (count($_sortarray) > 0) {
			$i = 0;
			$sum = 0;
			for(reset($_sortarray); $key = key($_sortarray); next($_sortarray)) {
				if ($i < $config_album["number_top10"]) {
				$this->_top10_array[$key] = $_sortarray[$key];
				}
				$i++;
				$sum += $_sortarray[$key];
			}
		$aver = ($sum / count($_sortarray));
		$this->_top10_array["sum"] = $sum;
		$this->_top10_array["aver"] = $aver;
		$this->_top10_array["number"] = count($_sortarray);
		}
                return $this->_top10_array;
		}
	}

	function sort_hits()
	{
		/* sorts images corresponding to the iamgehits */
		$this->_read_log();
		$_sortarray = $this->_log_array;
		if (count($_sortarray) > 1) {
			arsort($_sortarray);
		}
		$this->_sorthits_array = array_keys($_sortarray);

		return $this->_sorthits_array;
	}

}


/* get the Album-hits of current Album */

function albumhits($current_album)
	{
	global $config;

	$hitcounter = new HitCounter($config["log_filename"]);
	// make sure that there is a leading and a trailing slash
	$selected_album_leadingslash = "/" . $current_album . "/";
	// change doubleslashes to singleslash
	$selected_album_singleslash = str_replace("//","/",$selected_album_leadingslash);

	return $hitcounter->read_hit($selected_album_singleslash);
}


?>
