<?php

/*
 * ***********************************************************************************************
 * Filename:     check_basic.inc.php
 * Module:       Checks
 * Subcategory:  Checks startup basics
 * Description:  Checks for existing / readable directories and basic files needed
 * ***********************************************************************************************
 *
 *
 * Project:     yappa-ng : yet another php photo album - next generation
 * Author:      Fritz Berger <wizard@zirkon.at>
 * Copyright:   2003 Fritz Berger
 * $Header: /cvsroot/yappa-ng/yappa-ng/check_basic.inc.php,v 1.10 2003/08/14 23:41:39 adoll Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You may contact the author of yappa-ng by e-mail at:
 * wizard@zirkon.at
 *
 * The latest version of yappa-ng can be obtained from:
 *    http://www.zirkon.at/zirkon/scripts/yappa-ng/yappa-ng_main_ger.html (german)
 *    http://www.zirkon.at/zirkon/scripts/yappa-ng/yappa-ng_main_eng.html (english)
 *
 * ***********************************************************************************************
 * ***********************************************************************************************
 * ***                                                                                         ***
 * ***   yappa-ng is based on "YAPPA v1.7 devel - 22.April 2002"                               ***
 * ***                         released on http://sourceforge.net/projects/yappa/              ***
 * ***   YAPPA is Copyright Federico 'pix' Feroldi (pix@pixzone.com)                           ***
 * ***   YAPPA is released under the GNU GPL                                                   ***
 * ***                                                                                         ***
 * ***********************************************************************************************
 * ***********************************************************************************************
 */


// include_once('config.inc.php');

/* Suppress the warnings of undefined variables 
 * --------
 * Tip from the usercomments in the php-manual on http://www.php.net/manual/en/function.error-reporting.php:
 * --------
 * yappa-ng default setting = 2: getting all errors reported - but NOT the notices
 * (message: "undefined index xyz" if a $HTTP_POST/GET_VAR["xyz"] is not set at this moment (or similar) gets supressed!)
 */

switch ($config["error_rep"]) {
	case 1:
		$old_error_reporting = error_reporting(E_ALL);
		break;
	case 2:
		$old_error_reporting = error_reporting(E_ALL & ~(E_NOTICE));
		break;
	case 3:
		$old_error_reporting = error_reporting(E_ALL & ~(E_WARNING | E_NOTICE));
		break;
	default:
		$old_error_reporting = error_reporting(E_ALL & ~(E_NOTICE));
		break;
}




/* Standardtext appended to every errormsg */
$refer_to = "<br>For Details please refer to the file called <b>ERRORCODES</b> <br> in the /docs subdirectory of the yappa-ng distribution!<br>";

 
/* =======================
 *
 *  FATALS
 *
 * ======================= */

/* Test for main configuration file config.inc.php in the yappa-ng program directory
 * yappa-ng ERRORCODE 010
 * FATAL
 */
if (!file_exists('config.inc.php')) {
	print "<br>yappa-ng_ERROR_010: config.inc.php missing!!!";
	print $refer_to;
	$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . "yappa-ng_ERROR_010: config.inc.php missing!!!" . "\n";
	@error_log($log_msg, 3, "yappa-ng.error.log");
	die;
}

/* Test for config-variable "photo_root"
 * yappa-ng ERRORCODE 020 
 * FATAL
 */
if (!is_dir($config['photo_root'])) {
	print "<br>yappa-ng_ERROR_020: photo_root not set!!!";
	print $refer_to;
	$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . "yappa-ng_ERROR_020: photo_root is no directory or not readable  (" . $config['photo_root'] . ")!!!" . "\n";
	@error_log($log_msg, 3, "yappa-ng.error.log");
	die;
}

/* Test for album config-file config_album.inc.php in "photo_root"
 * yappa-ng ERRORCODE 021 
 * FATAL
 */
$file = $config['photo_root'] . "/config_album.inc.php";
if (!file_exists($file) || !is_readable($file) || !is_writeable($file)) {
	print "<br>yappa-ng_ERROR_021: no config_album.inc.php in photo_root!";
	print $refer_to;
	$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . "yappa-ng_ERROR_021: no config_album.inc.php in photo_root (" . $config['photo_root'] . ")!" . "\n";
	@error_log($log_msg, 3, "yappa-ng.error.log");
	die;
}


 
/* =======================
 *
 *  SEVERE WARNINGS
 *
 * ======================= */

/* IF ImageMagick is selected (recommended):
 * Test for convert utility
 * yappa-ng ERRORCODE 310 
 * WARNING
 */
if ($config['send_warnings'] != "no") {
	if ($config['image_module'] == "magick" ) {
		if (!file_exists($config['convert_path'])) {
			print "<br>yappa-ng_ERROR_310: convert utility not found!";
			print $refer_to;
			$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . "yappa-ng_ERROR_310: convert utility not found (" . $config['convert_path'] . ") !" . "\n";
			@error_log($log_msg, 3, "yappa-ng.error.log");
		}
	}
}

/* IF image caching is enabled (recommended):
 * Test for the cache directory
 * yappa-ng ERRORCODE 320 
 * WARNING
 */
if ($config['send_warnings'] != "no") {
	if ($config['resize_cache']) {
		if (!is_dir($config['cache_root'])) {
			print "<br>yappa-ng_ERROR_320: cache_root not found!";
			print $refer_to;
			$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . "yappa-ng_ERROR_320: cache_root not found (" . $config['cache_root'] . ") !" . "\n";
			@error_log($log_msg, 3, "yappa-ng.error.log");
		}
	}
}


/* =======================
 *
 *  WARNINGS
 *
 * ======================= */

/* Test for file cannot-delete-album in the photo_root:
 * yappa-ng ERRORCODE 510 
 * WARNING
 */
$file = $config['photo_root'] . "/cannot-delete-album";
if ($config['send_warnings'] != "no") {
	if (!file_exists($file)) {
		print "<br>yappa-ng_ERROR_510: no file cannot-delete-album in photo_root!";
		print $refer_to;
		$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . "yappa-ng_ERROR_510: no file: " . $file . "!" . "\n";
		@error_log($log_msg, 3, "yappa-ng.error.log");
	}
}

/* Test for file index.html in the photo_root:
 * yappa-ng ERRORCODE 511 
 * WARNING
 */
$file = $config['photo_root'] . "/index.html";
if ($config['send_warnings'] != "no") {
	if (!file_exists($file)) {
		print "<br>yappa-ng_ERROR_511: no index.html in photo_root!";
		print $refer_to;
		$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . "yappa-ng_ERROR_511: no index.html in photo_root ( no " . $file . " )!" . "\n";
		@error_log($log_msg, 3, "yappa-ng.error.log");
	}
}

/* Test for file hits.log in the yappa-ng program directory:
 * yappa-ng ERRORCODE 520 
 * WARNING
 */
if ($config['send_warnings'] != "no") {
	if (!file_exists("./hits.log") || !is_readable("./hits.log") || !is_writeable("./hits.log")) {
		print "<br>yappa-ng_ERROR_520: your hits.log for the album-counter not writable!";
		print $refer_to;
		$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . "yappa-ng_ERROR_520: your hits.log for the album-counter not writable !!" . "\n";
		@error_log($log_msg, 3, "yappa-ng.error.log");
	}
}

/* ==============================
 *
 *  ALBUM Specific Basic Check
 *
 * ============================== */


// read the album configuration of selected album!

if (isset($HTTP_GET_VARS["album"])) {
	$l_album = rawurldecode($HTTP_GET_VARS["album"]);
} else {
	$l_album = "";
}
$logfilename = singleslash($config["photo_root"] . "/" . $l_album . "/config_album.inc.php");

if(!file_exists($logfilename) || !is_readable($logfilename) || !$fd = fopen($logfilename, "r")) {
	print "ERROR!! File config_album.inc.php for album " . $l_album . " MISSING!!!!!";
	$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . "File config_album.inc.php for album " . $logfilename . " MISSING!!!!!" . "\n";
	@error_log($log_msg, 3, "yappa-ng.error.log");
} else {
	flock($fd, 1); // get a shared lock
	$data = fread($fd, filesize($logfilename));
	flock($fd, 3); // release the lock
	fclose($fd);
	$config_album = unserialize( stripslashes($data));  //strip slashes to prevent 'magic quotes' problems on some servers
}


/* If the hits.log exists and is readable and is writebale and at least one hit is logged!: everything ok  :-)
 * if NOT:
 *         - overrule Top10: switch OFF TopX
 *         - overrule "show hitcounters": switch off "show hitcounters"
 *         - overrule sortorder "album-hits": sitch to sortorder "name"
 * All these options do make NO SENSE if there is no / no writeable hits.log!
 * log in the "yappa-ng.warnings.log" !!!
 */
if (isset($HTTP_GET_VARS["album"])) {
	$l_album = rawurldecode($HTTP_GET_VARS["album"]);
} else {
	$l_album = "";
}
$l_hitsfilename = singleslash($config["photo_root"] . "/" . $l_album . "/" . $config['log_filename']);
$hitlog_readable = true;
/* First check, if the hits.log exists.
 * If NOT - disable hitcounter & tp10 */
if(!file_exists($l_hitsfilename) || !is_readable($l_hitsfilename)|| !is_writeable($l_hitsfilename)) {
	$hitlog_readable = false;
	if ($config_album["use_hitlog"] == "yes") {
		$config_album["use_hitlog"] = "no";
		$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . $l_hitsfilename . " is not read/writeable or does not exist. The display of the hitcounter in 'Single Image View' is switched off!!" . "\n";
		@error_log($log_msg, 3, "yappa-ng.warning.log");
	}
	if ($config_album["show_image_hits"] == "yes") {
		$config_album["show_image_hits"] = "no";
		$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . $l_hitsfilename . " is not read/writeable or does not exist. The display of the hitcounter in 'Thumbnail Mode' is switched off!!" . "\n";
		@error_log($log_msg, 3, "yappa-ng.warning.log");
	}
	if ($config_album["use_top10"] == "yes") {
		$config_album["use_top10"] = "no";
		$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . $l_hitsfilename . " is not read/writeable or does not exist. The display of the 'TopX' is switched off" . "\n";
		@error_log($log_msg, 3, "yappa-ng.warning.log");
	}
	if ($config_album["mark_top10"] == "yes") {
		$config_album["mark_top10"] = "no";
		$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . $l_hitsfilename . " is not read/writeable or does not exist. The display of the 'TopX Images' in the 'Thumbnail Mode' is switched off" . "\n";
		@error_log($log_msg, 3, "yappa-ng.warning.log");
	}
	if ($config_album["sort_by"] == "hits") {
		$config_album["sort_by"] = "name";
		$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . $l_hitsfilename . " is not read/writeable or does not exist. The sortorder 'Sort by Hits' is changed to 'Sort by Name'" . "\n";
		@error_log($log_msg, 3, "yappa-ng.warning.log");
	}
} else {
	/* hits.log does exists. But are there hits in it or is it an empty hitlog?
	* If EMPTY - disable hitcounter & tp10 */
	$fd = fopen($l_hitsfilename, "r");
	flock($fd, 1); // get a shared lock
	$data = fread($fd, filesize($l_hitsfilename));
	flock($fd, 3); // release the lock
	fclose($fd);
	$l_log_array = unserialize( stripslashes($data)); // stripslashes to prevent 'magic quotes' problems on some servers
	if (!(is_array($l_log_array))) {
		$hitlog_readable = false;
		$config_album["use_hitlog"] = "no";
		$config_album["show_image_hits"] = "no";
		$config_album["use_top10"] = "no";
		$config_album["mark_top10"] = "no";
		if ($config_album["sort_by"] == "hits") {
			$config_album["sort_by"] = "name";
		}
		//$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "check_basic.inc.php" . "\t" . "At the moment there is NO hit in the Album: " . rawurldecode($HTTP_GET_VARS["album"]) . ". Top10, sortorder 'hits', use image hits,.. have been disabled FOR THIS RUN!" . "\n";
		//@error_log($log_msg, 3, "yappa-ng.warning.log");
	}
}

?>
