<?php


/*
 * ***********************************************************************************************
 * Filename:     common.php
 * Module:       functions
 * Subcategory:  -
 * Description:  common used functions - a conglomeration for different needs
 * ***********************************************************************************************
 *
 *
 * Project:     yappa-ng : yet another php photo album - next generation
 * Author:      Fritz Berger <wizard@zirkon.at>
 * Copyright:   2003 Fritz Berger
 * $Header: /cvsroot/yappa-ng/yappa-ng/common.php,v 1.27 2003/08/13 20:36:17 zirkon13 Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You may contact the author of yappa-ng by e-mail at:
 *    wizard@zirkon.at
 *
 * The latest version of yappa-ng can be obtained from:
 *    http://www.zirkon.at/zirkon/scripts/yappa-ng/yappa-ng_main_ger.html (german)
 *    http://www.zirkon.at/zirkon/scripts/yappa-ng/yappa-ng_main_eng.html (english)
 *
 * ***********************************************************************************************
 * ***********************************************************************************************
 * ***                                                                                         ***
 * ***   yappa-ng is based on "YAPPA v1.7 devel - 22.April 2002"                               ***
 * ***                         released on http://sourceforge.net/projects/yappa/              ***
 * ***   YAPPA is Copyright Federico 'pix' Feroldi (pix@pixzone.com)                           ***
 * ***   YAPPA is released under the GNU GPL                                                   ***
 * ***                                                                                         ***
 * ***********************************************************************************************
 * ***********************************************************************************************
 */


/*
 * makeselflink($params)
 * makes a link back to yappa-ng passing the parameters from $params array
 * Added 04. Aug.2003: makes sure that the "index.php" is always in the url for sites where index.php is not "autostarting"
 *
 */
function makeselflink($params)
{
	global $HTTP_SERVER_VARS;
	$url = getenv("PHP_SELF") ? getenv("PHP_SELF") : $HTTP_SERVER_VARS["PHP_SELF"];

	if(is_array($params)) {
		if (eregi("index\.php$",$url)) {
			$url .= "?";
		} else {
			$url .= "index.php?";
		}
		while(list($key, $value) = each($params)) {
			$url .= $key . "=" . rawurlencode($value) . "&";
		}
	}

    return $url;
}

/*
 * makeshowlink($image_name, $album_name, $size)
 * makes a link to the show script
 *
 * This is the central function for imagedisplay!!!
 * If you have trouble with the show script (your webserver does not allow calls with trailing path_infos)
 * there is an alternate display option.
 * Read the details in the FAQ !
 * Changed in release 2.1.0 (01. Aug. 2003):
 *    Old: Info transmitted as "Trailing PathInfo" PATH_INFO (which needs AcceptPathInfo On in apache setup and ??? in IIS setup)
 *    New: Info transmitted as regular QUERY_STRING. This should anhance the OS / Webserver compatibility.
 * Since I do not exactly know if really every OS / Webbrowser is happy with this setup, i will leave the old as option in it!
 * Corresponding sourcecodechange in show.php.
 * Thank you Marc Andrew Pawlowsky for the complete working sourcecode-snipped in Bug Tracker on sourceforge.net.
 */
function makeshowlink($image_name, $album_name, $size)
{
	global $define, $config;
	global $HTTP_SERVER_VARS;


	/* is the 'alternate image display' enabled? (should be NOT in 98% of all setups!!)
	 * should not be necessary since the new method! */
	if ($config['i_see_no_images'] != "yes") {
		/* should the "old" version of the call to the show.php function be used? */
		if ($config['use_pathinfo'] != "yes") {
			/* New method to call the show.php script */
			$result =  singleslash($define["resize_script"] . '?' .'size=' . $size . '&amp;album_name=' . rawurlencode($album_name) . '&amp;obj_name=' . rawurlencode($image_name));
		} else {
			/* Old method to call the show.php script. Only for my own reassureance kept in the sourcecode :-) */
			$result = singleslash($define["resize_script"] . "/$size/" . $album_name . '/' . htmlentities(rawurlencode($image_name)));
		}
	} else {
		/* Alternate display method enabled! */
		$config['cache_root'] = singleslash($config['yappa-ng_path'] . "/cache/");
		$l_displaypath = include("image_alternate_display.inc.php");
		$uri_base = str_replace("index.php","",$HTTP_SERVER_VARS['PHP_SELF']);
		$path = str_replace($config['yappa-ng_path'],"",$l_displaypath);
		$l_displaypath =singleslash($uri_base . $path);
		$result = $l_displaypath;
	}
	return $result;

}

/*
 * get_image_mime($filename)
 * returns the mime type of the filename
 * returns null if the extension is unknown
 *
 */
function get_image_mime($filename)
{
	global $define;

	if(preg_match('/\.([^\.]+)$/', $filename, $matches)) {
		if (isset($define["image_mime"][strtolower($matches[1])])) {
			if($mime = $define["image_mime"][strtolower($matches[1])]) {
				return $mime;
			}
		}
	}
	return;
}

/*
 * calc_resized_geometry($src_width, $src_height, $dst_width, $dst_height)
 * compute the resized image geometry with the same aspect ratio of the
 * source image
 * returns an array with width and height
 */
function calc_resized_geometry($src_width, $src_height, $dst_width, $dst_height)
{

	$ratio_width = $dst_width / $src_width;
	$ratio_height = $dst_height / $src_height;

	$ratio = ($ratio_width < $ratio_height) ? $ratio_width : $ratio_height;

	return array($src_width * $ratio, $src_height * $ratio);
}

/*
 * calc_resized_width_height($source_image_path, $resize_size)
 * compute the resized image geometry width and height
 * with the same aspect ratio of the source image
 * returns an array with width and height
 * This is NOT needed for the resize calculation but ONLY for the width/height parameters in the HTML img-tag or table parameters
 *    to prevent resizeing during page-buildup.
 * Until 2.0.0 all sizes of thumbnails,... where taken from the size of the CACHED image.
 * But if there is no chached image there are no values.  :-(
 * This function changes this deficit. ;-)
 */
function calc_resized_width_height($source_image_path, $resize_size)
{
	global $config;

	// does this image exist (should do - came through some prior checks - but you never know!)
	if (file_exists($source_image_path)) {
		// ok - get original image sizes
		$l_orig_image_size = @getimagesize($source_image_path, $info);
		$l_orig_width = $l_orig_image_size[0];
		$l_orig_height = $l_orig_image_size[1];

		// get the wanted resize-otpion
		if ($resize_size == "original") {
			// its original - NO resize. take the original size as dst size - will give ratio=1
			$dst_width = $l_orig_width;
			$dst_height = $l_orig_height;
		} elseif ($resize_size == "thumbnail") {
			// its thumbnail - ok we go with thumbnail
			$dst_width = $config['thumb_width'];
			$dst_height = $config['thumb_height'];
		} else {
			// ok - a resize-option
			list($dst_width, $dst_height) = split('x', $resize_size);
		}
		/* we now have determined the dst-width/dst-height
		 * let's calculate the ratio 
		 * Plausibilitycheck contributed by Marc Andrew Pawlowsky */
		if (($l_orig_width != 0) && ($l_orig_height != 0) ) {
			$ratio_width = $dst_width / $l_orig_width;
			$ratio_height = $dst_height / $l_orig_height;

			$ratio = ($ratio_width < $ratio_height) ? $ratio_width : $ratio_height;
		} else {
			$ratio = 1;
		}

		// finished! make integers out of the values and make width/height HTML-parameter out of the values
		$l_resize_width = intval(($l_orig_width * $ratio));
		$l_resize_height = intval(($l_orig_height * $ratio));
		$l_resize_html = " width='" . $l_resize_width . "' height='" . $l_resize_height . "' ";
	} else {
		/* image does not exist (should be NEVER the case - but just to make 'save code')
		 * For this special case I make special size to help debug it!
		 * If you EVER get an image resized to 321x321 PLEASE email me!!!
		 * As stated: this should never happen!! */
		$l_resize_width = 321;
		$l_resize_height = 321;
		$l_resize_html = " width='" . $l_resize_width . "' height='" . $l_resize_height . "' ";
	}



	return array($l_resize_width, $l_resize_height, $l_resize_html);
}



/* deletes all cached resized copies of a selected image
 * if cache is enabled
 * $src_album is the album-url
 * $src_image is the imagename
 */
function delimage_cache($src_album, $src_image) {
	global $config;

	/* if the option resize_cache is enabled in the config.inc.php, then go on */
	if ($config['resize_cache']) {
		$l_resize = $config['resize_options'];
		$l_resize[] = $config['resize_mini'];
		$l_resize[] = "thumbnails";

		/* look for each defined resize-size if there is an resized image and delete it */
		foreach ($l_resize as $l_size) {
			if (file_exists($config["cache_root"] . "/" . $src_album . "/_" . $l_size . "/" . $src_image )) {
				unlink ($config["cache_root"] . "/" . $src_album . "/_" . $l_size . "/" . $src_image );
			}
		}
	}
}



/* checks, if the cache-path is existing.
 * if not, it gets created - directory after directory the complete hierarchy
 */
function makepath($root, $subpath, $mode = '')
{
    global $define;
   
    /* if no specific mode is passed to the function
     * the mode from the define-variable is taken.
     * mode means the unix filepermission mode: eg 777 = rwxrwxrwx
     */
    if($mode == '') {
        $mode = $define['mkdir_mode'];
    }

    $dir_array = split('/', $subpath);
    $path = $root;
    foreach($dir_array as $dir) {
        $path .= "/$dir";
        if(!file_exists($path)) {
    	    if(!@mkdir($path, $mode)) {
                return;
            }
        }
    }
    return $path;
}

/* This function checks if the image is already in the cache-directory.
 * if it is it returns the path
 * if not it creates the cached resized image 
 */
function update_cache($album_name, $obj_name, $resize)
{
	global $config, $define;

	$album_root = singleslash($config["photo_root"] . '/' . $album_name);
	if(!is_dir($album_root)) {
		$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "common.php" . "\t" . "album root [$album_root] does not exist!" . "\n";
		@error_log($log_msg, 3, "yappa-ng.error.log");
		die("album does not exist [$album_root]");
	}

	$obj_path = singleslash($album_root . '/' . $obj_name);

	if($config["cache_root"]) {
		$cache_root = $config["cache_root"];
		if(!@file_exists($cache_root)) {
			if(!@mkdir($cache_root, $define["mkdir_mode"])) {
				//$cache_root = $config["photo_root"];
			}
		}
	} else {
		//$cache_root = $config["photo_root"];
	}

	if($resize == 'thumbnail') {
		$cache_dir = $define[thumbnails_dirname];
		$resize_width = $config["thumb_width"];
		$resize_height = $config["thumb_height"];
	} else {
		$cache_dir = $resize;
		list($resize_width, $resize_height) = split('x', $resize);
	}

	$cache_dir = singleslash($album_name . '/_' . $cache_dir);
	$image_cached = singleslash($cache_root . '/' . $cache_dir . '/' . $obj_name);

	if(!file_exists($image_cached) || (filemtime($obj_path) > filemtime($image_cached))) {
		if(!@file_exists($resize_path = $cache_root . '/' . $cache_dir)) {
			if(!makepath($cache_root, $cache_dir)) {
				$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "common.php" . "\t" . "cannot create dir of resize path: [$resize_path]" . "\n";
				@error_log($log_msg, 3, "yappa-ng.error.log");
				die("cannot create dir [$resize_path]");
			}
		}

		$cimage = new ImageResize($obj_path);
		$cimage->set_params(
			array(
			"resize_width" => $resize_width,
			"resize_height" => $resize_height,
			"quality" => $config["resize_quality"]
			)
		);
		$cimage->convert($image_cached);
	}
	return $image_cached;
}


/* previous / next page
 * calculates the necessary limits, values and links
 * and stores them in the prevnext-array
 */
function page_pn() {

	global $config, $config_album;
	global $album, $selected_album;
	global $HTTP_GET_VARS;

	$prevnext[] = array();
	/* make sure the number of images shown on one page is initialised */
	if ($config_album['mini_images_per_page'] < 1) {
		$config_album['mini_images_per_page'] = 25;
	}
	/* look how many images are in this album */
	$l_number_of_images = $album->count_images();
	$l_mini_images_per_page = $config_album['mini_images_per_page'];

	$l_current_page = floor($HTTP_GET_VARS["page"]);
	$l_admintask = $HTTP_GET_VARS['admintask'];

	$l_max_page = ceil($l_number_of_images / $l_mini_images_per_page);
	if(($l_current_page < 1)) {
		$l_current_page = 1;
	} elseif($l_current_page > $l_max_page) {
		$l_current_page = $l_max_page;
	}
	$prevnext['mini_start'] = (($l_current_page - 1) * $l_mini_images_per_page);
	$prevnext['mini_end'] = min(($l_current_page * $l_mini_images_per_page), $l_number_of_images);




	/* calculate page navigation links */

	$l_navlink_pages = floor($config["navlink_pages"] / 2);

	if($l_current_page <= $l_navlink_pages) {
		$l_start_page = 1;
		$l_end_page = $l_start_page + $l_navlink_pages * 2;
	} elseif($l_current_page > ($l_max_page - $l_navlink_pages)) {
		$l_end_page = $l_max_page;
		$l_start_page = $l_end_page - $l_navlink_pages * 2;
	} else {
		$l_start_page = $l_current_page - $l_navlink_pages;
		$l_end_page = $l_current_page + $l_navlink_pages;
	}

	if($l_start_page < 1) {
		$l_start_page = 1;
	}

	if($l_end_page > $l_max_page) {
		$l_end_page = $l_max_page;
	}

	$l_back_page = $l_start_page - 1;
	$l_forw_page = $l_end_page + 1;

	// $prevnext['navlinks'] = array();
	for($i = $l_start_page; $i <= $l_end_page; $i++) {
		$prevnext['navlinks'][] = array(
			"page" => $i,
			"link" => makeselflink(array("album" => $selected_album, "adminlogin" => "yes", "admintask" => $l_admintask, "page" => $i)),
			"selected" => ($i == $l_current_page) ? 1 : 0,
		);
	}

	if($l_start_page > 1) {
		$prevnext['navlink_back'] = makeselflink(array("album" => $selected_album, "adminlogin" => "yes", "admintask" => $l_admintask, "page" => $l_back_page));
	}

	if($l_end_page < $l_max_page) {
		$prevnext['navlink_forw'] = makeselflink(array("album" => $selected_album, "adminlogin" => "yes", "admintask" => $l_admintask, "page" => $l_forw_page));
	}

	if($l_current_page > 1) {
		$prevnext['navlink_previous'] = makeselflink(array("album" => $selected_album, "adminlogin" => "yes", "admintask" => $l_admintask, "page" => ($l_current_page - 1)));
	}

	if($l_current_page < $l_max_page) {
		$prevnext['navlink_next'] = makeselflink(array("album" => $selected_album, "adminlogin" => "yes", "admintask" => $l_admintask, "page" => ($l_current_page + 1)));
	}


	return $prevnext;
}


function upd($album_name, $obj_name, $resize)
{
	global $config, $define;
	global $HTTP_SERVER_VARS;

	$album_root = singleslash($config["photo_root"] . '/' . $album_name);
	if(!is_dir($album_root)) {
		$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "common.php" . "\t" . "album root [$album_root] does not exist!" . "\n";
		@error_log($log_msg, 3, "yappa-ng.error.log");
		//die("album root doesn't exists [$album_root]");
	}

	$obj_path = singleslash($album_root . '/' . $obj_name);


	if($config["cache_root"]) {
		$cache_root = $config["cache_root"];
		if(!@file_exists($cache_root)) {
			if(!@mkdir($cache_root, $define["mkdir_mode"])) {
				$cache_root = $config["photo_root"];
			}
		}
	} else {
		$cache_root = $config["photo_root"];
	}

	if($resize == 'thumbnail') {
		$cache_dir = $define['thumbnails_dirname'];
		$resize_width = $config["thumb_width"];
		$resize_height = $config["thumb_height"];
	} else {
		$cache_dir = $resize;
		list($resize_width, $resize_height) = split('x', $resize);
	}

	$cache_dir = $album_name . "/_" . $cache_dir;
	$image_cached = $cache_root . "/" . $cache_dir . "/" . $obj_name;

	if(!file_exists($image_cached) || (filemtime($obj_path) > filemtime($image_cached))) {
		if(!@file_exists($resize_path = "$cache_root/$cache_dir")) {
			if(!makepath($cache_root, $cache_dir)) {
				$log_msg = date("d.m.y-H:i") . "\t" . __LINE__ . "\t" . "common.php" . "\t" . "cannot create dir of resize path: [$resize_path]" . "\n";
				@error_log($log_msg, 3, "yappa-ng.error.log");
				//die("cannot create dir [$resize_path]");
			}
		}

		$cimage = new ImageResize($obj_path);
		$cimage->set_params(
			array(
			"resize_width" => $resize_width,
			"resize_height" => $resize_height,
			"quality" => $config["resize_quality"]
			)
		);
		$cimage->convert($image_cached);
	}
	return $image_cached;
}


/* I'm lazy and I know there are parts with double-slashes instead of the one needed slash!
 * It occurs in paths and rawurlencoded paths!
 * This function takes care of this lazyness */
function singleslash($path) {
	$l_single_slash = str_replace("///","/",$path);
	$path = str_replace("//","/",$l_single_slash);

	$l_single_slash = str_replace("%2F%2F%2F","%2F",$path);
	$single_slash = str_replace("%2F%2F","%2F",$l_single_slash);
	return $single_slash;
}


?>
