<?php

/*
 * ***********************************************************************************************
 * Filename:     image-gd.class.php
 * Module:       image-gd Class
 * Subcategory:  -
 * Description:  provides interaction with the gd library
 * ***********************************************************************************************
 *
 *
 * Project:     yappa-ng : yet another php photo album - next generation
 * Author:      Fritz Berger <wizard@zirkon.at>
 * Copyright:   2003 Fritz Berger
 * $Header: /cvsroot/yappa-ng/yappa-ng/image-gd.class.php,v 1.6 2003/06/15 17:07:49 zirkon13 Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You may contact the author of yappa-ng by e-mail at:
 *    wizard@zirkon.at
 *
 * The latest version of yappa-ng can be obtained from:
 *    http://www.zirkon.at/zirkon/scripts/yappa-ng/yappa-ng_main_ger.html (german)
 *    http://www.zirkon.at/zirkon/scripts/yappa-ng/yappa-ng_main_eng.html (english)
 *
 * ***********************************************************************************************
 * ***********************************************************************************************
 * ***                                                                                         ***
 * ***   yappa-ng is based on "YAPPA v1.7 devel - 22.April 2002"                               ***
 * ***                         released on http://sourceforge.net/projects/yappa/              ***
 * ***   YAPPA is Copyright Federico 'pix' Feroldi (pix@pixzone.com)                           ***
 * ***   YAPPA is released under the GNU GPL                                                   ***
 * ***                                                                                         ***
 * ***********************************************************************************************
 * ***********************************************************************************************
 */

include_once("common.php");
 	
class ImageUtil
{
	var $_can_truecolor;	
	
	function ImageUtil(&$config)
	{
		/* sometimes the autodetect does not work on some systems using GD < 2.0.
		* here is a way to not try to autodetect but to set it in the config! */
		if ($config['gd_truecolor'] != "no" ) {
			$this->_can_truecolor = (function_exists("ImageCreateTruecolor") && function_exists("ImageCopyResampled")) ? 1 : 0;
		} else {
			$this->_can_truecolor = 0;
		}
	}

	function resize($src, $dst, $params)
	{
		$im_mime = get_image_mime($src);

		switch($im_mime) {
			case "image/jpeg":
				ImageJPEG($this->_resizeimage(ImageCreateFromJPEG($src), $params["resize_width"], $params["resize_height"]), $dst, $params["quality"]);
				break;
			case "image/gif":
				ImageGIF($this->_resizeimage(ImageCreateFromGIF($src), $params["resize_width"], $params["resize_height"]), $dst);
				break;
			case "image/png":
				ImagePNG($this->_resizeimage(ImageCreateFromPNG($src), $params["resize_width"], $params["resize_height"]), $dst);
				break;
			default:
				return;
		}
	}

	/* rotate will only give an error and do nothing if you use PHP < 4.3.0 !!! 
	 * Sorry! */
	function rotate($src, $params)
	{
		$im_mime = get_image_mime($src);
		$dst = $src;

		switch($im_mime) {
			case "image/jpeg":
				ImageJPEG($this->_rotateimage(ImageCreateFromJPEG($src), $params["rotate"]), $dst, $params["quality"] );
				break;
			case "image/gif":
				ImageGIF($this->_rotateimage(ImageCreateFromGIF($src), $params["rotate"]), $dst  );
				break;
			case "image/png":
				ImagePNG($this->_rotateimage(ImageCreateFromPNG($src), $params["rotate"]), $dst);
				break;
			default:
				return;
		}
	}

	function _resizeimage($im_src, $dst_width, $dst_height)
	{
		$src_width = ImageSX($im_src);
		$src_height = ImageSY($im_src);

		list($res_width, $res_height) = calc_resized_geometry($src_width, $src_height, $dst_width, $dst_height);

		$im_dst = $this->_can_truecolor ?
		ImageCreateTruecolor($res_width, $res_height) :
		ImageCreate($res_width, $res_height);

		$this->_can_truecolor ?
		ImageCopyResampled($im_dst, $im_src, 0, 0, 0, 0, $res_width, $res_height, $src_width, $src_height) :
		ImageCopyResized($im_dst, $im_src, 0, 0, 0, 0, $res_width, $res_height, $src_width, $src_height);

		return $im_dst;
	}

	function _rotateimage($im_src, $rotate)
	{

		$im_dst = ImageRotate($im_dst, $rotate, 0);

		return $im_dst;
	}

}

?>
